
import java.util.*;
import java.io.*;
/**
 * Verwaltet den Garten
 * 
 * @author Albert Wiedemann 
 * @version 1.0
 */
class Garten
{
    /** Feld der Gartenelemente */
    private ArrayList<ArrayList<GartenElement>> elemente;

    /**
     * Legt den Garten an.
     * @param dateiName Name der Datei mit der Beschreibung des Gartens
     * @param graph der Graph, der den Garten repräsentiert.
     */
    Garten (String dateiName, GraphMatrix graph)
    {
        elemente = new ArrayList<ArrayList<GartenElement>>();
        if (Lesen(dateiName))
        {
            GraphErzeugen(graph);
        }
    }
    
    /**
     * Liest die Beschreibungs des Gartens aus einer Datei ein
     * @param dateiName Name der Datei
     * @return wahr, wenn die Datei gelesen werden konnte
     */
    private boolean Lesen (String dateiName)
    {
        try
        {
            int x;
            int y;
            BufferedReader eingabe = new BufferedReader(new FileReader(dateiName));
            y = 0;
            while (true)
            {
                String zeile = eingabe.readLine();
                if (zeile == null)
                {
                    break;
                }
                elemente.add(new ArrayList<GartenElement>());
                
                x = 0;
                for (char ch: zeile.toLowerCase().toCharArray())
                {
                    GartenElement e;
                    if (ch == 'h')
                    {
                        e = new Hindernis(x, y);
                    }
                    else if (ch == 'r')
                    {
                        e = new Rasen(x, y);
                    }
                    else
                    {
                        e = new GartenElement(x, y);
                        e.darstellung.FarbeSetzen("rot");
                    }
                    elemente.get(y).add(e);
                    x += 1;
                }
                y += 1;
            }
            eingabe.close();
            return true;
        }
        catch (Exception e)
        {
            Text t = new Text();
            t.PositionSetzen(10, Zeichenfenster.MalflächenHöheGeben() - 30);
            t.TextSetzen("Fehler beim Lesen der Datei '" + dateiName + "': " + e.getMessage());
            return false;
        }
    }
    
    /**
     * Erzeugt die Graphendarstellung
     * @param graph das Objekt zur Verwaltung des Graphen
     */
    void GraphErzeugen(GraphMatrix graph)
    {
        for (int y = 0; y < elemente.size(); y += 1)
        {
            for (int x = 0; x < elemente.get(y).size(); x += 1)
            {
                if (elemente.get(y).get(x) instanceof Rasen)
                {
                    graph.KnotenEinfügen(x, y);
                    if ((y > 0) && (elemente.get(y - 1).get(x) instanceof Rasen))
                    {
                        graph.KanteEinfügen(x, y - 1, x, y);
                    }
                    if ((x > 0) && (elemente.get(y).get(x - 1) instanceof Rasen))
                    {
                        graph.KanteEinfügen(x - 1, y, x, y);
                    }
                }
            }
        }
    }
}
