
import java.util.ArrayList;
/**
 * Verwaltet einen ungerichteten, gewichteten Graphen mittels Adjazenzmatrix
 * 
 * @author Albert Wiedemann 
 * @version 1.0
 */
class GraphMatrix
{
    /** Feld der Knoten des Graphen */
    private ArrayList<Knoten> knoten;   
    /** 2-dim Feld der Adjazenzmatrix */
    private ArrayList<ArrayList<Integer>> matrix;
    /** Feld der Kantensymbole des Graphen */
    private ArrayList<KantenSymbol> kanten;   
    
    /**
     * Baut die Datenstruktur auf
     */
    GraphMatrix ()
    {
        knoten = new ArrayList<Knoten>();
        matrix = new ArrayList<ArrayList<Integer>>();
        kanten = new ArrayList<KantenSymbol>();
    }
    
    /**
     * Einfügen eines neuen Knoten in den Graphen
     * @param x x-Koordinate des Knotens
     * @param y y-Koordinate des Knotens
     */
    void KnotenEinfügen(int x, int y)
    {
        knoten.add(new Knoten(x, y));
        for (int index = 0; index < matrix.size(); index++)
        {
            matrix.get(index).add(new Integer(-1));
        }
        matrix.add(new ArrayList<Integer>());
        for (int index = 0; index < matrix.size() - 1; index++)
        {
            matrix.get(matrix.size()-1).add(new Integer(-1));
        }
        matrix.get(matrix.size()-1).add(new Integer(0));
    }

    /**
     * Gibt die interne Nummer des Knotens
     * Wenn ein Knoten mit diesen Koordinaten nicht bekannt ist wird -1 zurückgegeben
     * @param x x-Koordinate des Knotens
     * @param y y-Koordinate des Knotens
     * @return Indexnummer des Knotens im Knotenfeld; 0 <= res <= knoten.size()-1 bzw. -1
     */   
    private int KnotenNummerGeben(int x, int y)
    {
        for (int index=0; index < knoten.size(); index++)
        {
            Knoten k = knoten.get(index);
            if ((k.XGeben() == x) && (k.YGeben() == y))
            {
                return index;
            }
        }
        return -1;
    }

    /**
     * Gibt die interne Nummer des Knotens
     * @param k der Knoten
     * @return Indexnummer des Knotens im Knotenfeld; 0 <= res <= knoten.size()-1 bzw. -1
     */   
    private int KnotenNummerGeben(Knoten k)
    {
        for (int index=0; index < knoten.size(); index++)
        {
            if (knoten.get(index) == k)
            {
                return index;
            }
        }
        return -1;
    }

    /**
     * Einfügen einer Kante in den Graphen
     * Eine Kante ist durch einen Anfangsknoten und einen Endknoten festgelegt und hat eine Gewichtung
     * @param x1 x-Koordinate des Anfangsknotens
     * @param y1 y-Koordinate des Anfangsknotens
     * @param x2 x-Koordinate des Endknotens
     * @param y2 y-Koordinate des Endknotens
     */
    void KanteEinfügen(int x1, int y1, int x2, int y2)
    {
        int vonNummer, nachNummer;
        vonNummer = KnotenNummerGeben(x1, y1);
        nachNummer = KnotenNummerGeben(x2, y2);
        if ((vonNummer!=-1) && (nachNummer!=-1) && (vonNummer!=nachNummer))
        {
            matrix.get(vonNummer).set(nachNummer, 1);
            matrix.get(nachNummer).set(vonNummer, 1);
            kanten.add(new KantenSymbol(knoten.get(vonNummer).SymbolGeben(), knoten.get(nachNummer).SymbolGeben(), false, "", 3, "blau"));
        }
    }

    /**
     * Gibt die Anzahl der Knoten des Graphen zurück
     * @return  Anzahl der Knoten
     */   
    private int KnotenAnzahlgeben()
    {
        return knoten.size();
    }

    /**
     * Führt die Breitensuche zum Testen des Zusammenhangs durch
     * Falls am Ende alle Knoten besucht wurden, ist der Graph zusammenhängend
     * @return WAHR, wenn der Graph zusammenhängend ist.
     */
    boolean ZusammenhangTesten()
    {
    	//Nach Aufgabenstellung ergänzen
        return false;
    }
}
